#include <MIDI.h>
#include <math.h>
#include "Mytypes.h"


// Создаем  MIDI интерфейс
MIDI_CREATE_DEFAULT_INSTANCE();

//Конфигурация пинов
#define CPU_D_0 2        //Шина данных bit 0 
#define CPU_D_1 4        //Шина данных bit 1
#define CPU_D_2 13       //Шина данных bit 2
#define CPU_D_3 14       //Шина данных bit 3
#define CPU_D_4 16       //Шина данных bit 4
#define CPU_D_5 17       //Шина данных bit 5
#define CPU_D_6 18       //Шина данных bit 6
#define CPU_D_7 19       //Шина данных bit 7
#define SH_AN0 34        //Аналоговый пин для педалей и регулятора громкости.
#define SPBLANK 33       //Out Enable для U3 74HCT374 (матрица светодиодов) в состоянии LOW пишем данные.
#define SPEN_L 32        //Out Enable для U8 74HC245 (комутатор управления устройствами) в состоянии LOW пишем данные.
#define CPU_WR_BUF_L 27  //Выбор направления передачи для U6 74HC245 (комутатор шины данных) в состоянии LOW пишем данные, в состоянии HIGH читаем данные
#define SCAN_CS_L 26     //Out Enable для U6 74HC245 (комутатор шины данных) в состоянии LOW пишем или читаем данные.
#define SPRES_L 25       //Выбор столбца: Вход элементов "И" для U5 74HC164 (клавиатура). Так же вход элементов "И" для U1 74HC164 (матрица светодиодов и кнопок).
#define SPREN_SW_L 23    //Out Enable для U4 74LVC245A в состоянии LOW читаем данные (матрица кнопок).
#define SPWEN_LED_L 22   //CLK для U1 74HC164 (матрица светодиодов и кнопок). Так же CLK для U3 74HCT374 (матрица светодиодов).
#define SPREN_KY_L 21    //Out Enable для U12 74HC245 в состоянии LOW читаем данные. Так же CLK для U5, U8, U9, LOW->HIGH - сдвигаем регистр. (клавиатура)

//пин записи в 74НС4051 (0-7) шина CPU_D_0 - CPU_D_4. Выбор педали или регулятора громкости.
//На шине 0 - Регклятор громкости.
//На шине 3 - Педаль Sustain
//На шине 2 - Педаль Sustenuto
//На шине 1 - Педаль CCPeda
#define SPEN_AN_L 15 //CLK для U16 74HCT374 (управление мультиплексором U15 74hc4051). Использует 5 младших бит шины данных.

#define AN_VOLUME 0
#define AN_SUSTENUTO 2
#define AN_SUSTAIN 3
#define AN_CCPEDA 1

//General MIDI контроллеры
#define Pedal_Sustain 64
#define Pedal_Sostenuto 66
#define Pedal_Soft 67
#define Control_Volume 7

#define MY_PERIOD 1           // Период в мс для таймера tmr1


#define MIDI_CHANNEL 1        // Номер канала MIDI

uint8_t port_output = false;  //Состояние шины данных (запись/чтение)
keyp_t keys[88];
#define NUM_BANKS 11          //Количество банков (у нас 11 банков по 8 клавиш)

// Для сканирования банков
bank_t banks[NUM_BANKS];      //Текущие значения банков
bank_t prev_banks[NUM_BANKS]; //Предыдущие значения банков

void setup() {
	//Настройка портов управления на выход и уровень HIGH
  pinMode(SPBLANK,      OUTPUT);
  pinMode(SPEN_L,       OUTPUT);
  pinMode(CPU_WR_BUF_L, OUTPUT);
  pinMode(SCAN_CS_L,    OUTPUT);
  pinMode(SPRES_L,      OUTPUT);
  pinMode(SPREN_SW_L,   OUTPUT);
  pinMode(SPWEN_LED_L,  OUTPUT);
  pinMode(SPREN_KY_L,   OUTPUT);
  pinMode(SPEN_AN_L,    OUTPUT);
  digitalWrite(SPBLANK,      HIGH);
  digitalWrite(SPEN_L,       HIGH);
  digitalWrite(CPU_WR_BUF_L, HIGH);
  digitalWrite(SCAN_CS_L,    HIGH);
  digitalWrite(SPRES_L,      HIGH);
  digitalWrite(SPREN_SW_L,   HIGH);
  digitalWrite(SPWEN_LED_L,  HIGH);
  digitalWrite(SPREN_KY_L,   HIGH);
  digitalWrite(SPEN_AN_L,    HIGH);

  // Инициализация массива банков
  memset(banks, 0xff, sizeof(prev_banks));
  
  // Иницилизируем ноты с 21 для 88 клавиатуры
  for ( int key = 0; key < 88; key++) {
    keys[key].midi_note = 21 + key;
    keys[key].t = 0;
  }
  
  //Перед запуском следует отредактировать файл serialMIDI.h библиотеки MIDI Library v5.0.2
  //BaudRate установить на 115200 для использования Hairless MIDI
  //thruActivated установить в false так как функции приема MIDI сообщений не используем.
  MIDI.begin(MIDI_CHANNEL_OMNI);  // Прослушивать все входящие сообщения (оключил)
  delay(1000);
}


//чтение байта из шины
uint8_t read_port() {
  uint8_t data = 0;
  if (port_output){
    pinMode(CPU_D_0, INPUT);
    pinMode(CPU_D_1, INPUT);
    pinMode(CPU_D_2, INPUT);
    pinMode(CPU_D_3, INPUT);
    pinMode(CPU_D_4, INPUT);
    pinMode(CPU_D_5, INPUT);
    pinMode(CPU_D_6, INPUT);
    pinMode(CPU_D_7, INPUT);
    port_output = false;
  }

  data = digitalRead(CPU_D_7);
  data = data << 1;
  data = data | digitalRead(CPU_D_6);
  data = data << 1;
  data = data | digitalRead(CPU_D_5);
  data = data << 1;
  data = data | digitalRead(CPU_D_4);
  data = data << 1;
  data = data | digitalRead(CPU_D_3);
  data = data << 1;
  data = data | digitalRead(CPU_D_2);
  data = data << 1;
  data = data | digitalRead(CPU_D_1);
  data = data << 1;
  data = data | digitalRead(CPU_D_0);
  
  return data;
}

//звпись байта в шину
void write_port(uint8_t data) {
  if(!port_output){
    pinMode(CPU_D_0, OUTPUT);
    pinMode(CPU_D_1, OUTPUT);
    pinMode(CPU_D_2, OUTPUT);
    pinMode(CPU_D_3, OUTPUT);
    pinMode(CPU_D_4, OUTPUT);
    pinMode(CPU_D_5, OUTPUT);
    pinMode(CPU_D_6, OUTPUT);
    pinMode(CPU_D_7, OUTPUT);
    port_output = true;
  }
  
  digitalWrite(CPU_D_7, data & 0b10000000);
  digitalWrite(CPU_D_6, data & 0b01000000);
  digitalWrite(CPU_D_5, data & 0b00100000);
  digitalWrite(CPU_D_4, data & 0b00010000);
  digitalWrite(CPU_D_3, data & 0b00001000);
  digitalWrite(CPU_D_2, data & 0b00000100);
  digitalWrite(CPU_D_1, data & 0b00000010);
  digitalWrite(CPU_D_0, data & 0b00000001);
}

//чтение аналоговых датчиков, на вход принимает номер сенсора 0-3
uint16_t read_analog_sensor(uint8_t sensor) {
  uint16_t value = 0;   
  write_port(sensor);              //Пишем в шину данных номер входа для U15 74HC4051
  digitalWrite(CPU_WR_BUF_L, LOW); //Включаем U6 74HC245 на выход 
  digitalWrite(SCAN_CS_L, LOW);    //Включаем комутатор шины
  digitalWrite(SPEN_L, LOW);       //Включаем U8 74HC245 на выход  

  //Пишем в U16 данные для U15, просто дергаем CLK
  digitalWrite(SPEN_AN_L, LOW);  
  digitalWrite(SPEN_AN_L, HIGH); 
  delay(5); //ждем пока напряжение установится в стабильное состояние на выходе U15 74HC4051

  value = analogRead(SH_AN0);      //Читаем состояние датчика   
  
  //Конец чтения
  digitalWrite(SCAN_CS_L, HIGH);
  digitalWrite(CPU_WR_BUF_L, HIGH);  
  digitalWrite(SPEN_L, HIGH);

  if(sensor){
    //Если это педаль то приводим к виду нажатая или нет
    if(value > 100){
      value = 0;
    }else{
      value = 127;
    }
  }else{
    //Если это регулятор громкости то сужаем диапазон значений до 127
    value = map(value, 0, 4095, 0, 127);
  } 

  return value;
}



//Обрабатываем событие клавиш
void trigger(keyp_t *key, event_t event) {

  if(event == KEY_PRESSED) {
    key->state = KEY_IS_GOING_DOWN;
    
  } else if (event == KEY_DOWN) {
    key->state = KEY_IS_DOWN;
    MIDI.sendNoteOn(key->midi_note, 127- key->t, MIDI_CHANNEL);
    key->t = 0;
    
  } else if (event == KEY_RELEASED) {
    key->state = KEY_IS_GOING_UP;
    
  } else if ( event == KEY_UP) {
    key->state = KEY_IS_UP;
    MIDI.sendNoteOff(key->midi_note,  127 - key->t, MIDI_CHANNEL);
    key->t = 0;
    
  }
  
}



//замеряем скорость нажатия клавиатуры
void increment() {     
   for(int key = 0; key < 88; key++) {
     state_t state = keys[key].state;
     if(state == KEY_IS_GOING_UP || state == KEY_IS_GOING_DOWN) {
       keys[key].t++;
     }
   }
}

//Обрабатываем наджатие педалей и регулятор громкости
void footpedal() {  
  MIDI.sendControlChange(Pedal_Sustain, read_analog_sensor(AN_SUSTAIN), MIDI_CHANNEL);
  MIDI.sendControlChange(Pedal_Sostenuto, read_analog_sensor(AN_SUSTENUTO), MIDI_CHANNEL);
  MIDI.sendControlChange(Pedal_Soft, read_analog_sensor(AN_CCPEDA), MIDI_CHANNEL);
  MIDI.sendControlChange(Control_Volume, read_analog_sensor(AN_VOLUME), MIDI_CHANNEL);  
}


void clear_row() {
  //Устанавливаем бит для записи в сдвиговый регистр U9 74HC164 (выбор столбца);
  digitalWrite(SPRES_L, HIGH);  
  for(uint8_t i = 0; i < 24; i++){
    //Дергаем CLK U9 74HC164 
    digitalWrite(SPREN_KY_L, LOW);
    digitalWrite(SPREN_KY_L, HIGH);
  }  
}

//Выбор столбца для чтения, всего 22 столбца
void selects_row(uint8_t row) {   
  for(uint8_t i = 0; i < 24; i++){
    if(i == (24 - (row + 1))){
      digitalWrite(SPRES_L, LOW);
    }else{
      digitalWrite(SPRES_L, HIGH);      
    }
    //Дергаем CLK U9 74HC164 
    digitalWrite(SPREN_KY_L, LOW);
    digitalWrite(SPREN_KY_L, HIGH);
  }  
}

void scan() {

  digitalWrite(CPU_WR_BUF_L, HIGH);   //Включаем U6 74HC245 на вход
  digitalWrite(SPEN_L, LOW);          //Включаем U8 74HC245 на выход
  clear_row();

  // Сканируем и сохраняем банки
  for(int bank = 0; bank < NUM_BANKS; bank++) {
    prev_banks[bank] = banks[bank];   // Сохраняем предыдущее состояние
    
    
    selects_row(bank * 2);            // Выбирает нижнюю строку BR
    delayMicroseconds(10);            // Время дребезга
    digitalWrite(SPREN_KY_L, LOW);    //Включаем U12 74HC245 на выход
    digitalWrite(SCAN_CS_L, LOW);     //Включаем комутатор шины
    banks[bank].bottom = read_port(); //Чтаем шину   
    digitalWrite(SCAN_CS_L, HIGH);    //Отключаем комутатор шины данных U6 74HC245
    digitalWrite(SPREN_KY_L, HIGH);
    
    
    selects_row(bank * 2 + 1);        // Выбирает верхнюю строку MK
    delayMicroseconds(10);            // Время дребезга
    digitalWrite(SPREN_KY_L, LOW);    //Включаем U12 74HC245 на выход
    digitalWrite(SCAN_CS_L, LOW);     //Включаем комутатор шины
    banks[bank].top = read_port();    //Чтаем шину
    digitalWrite(SCAN_CS_L, HIGH);    //Отключаем комутатор шины данных U6 74HC245
    digitalWrite(SPREN_KY_L, HIGH);
  }
  digitalWrite(SPEN_L, HIGH);         //Отключаем комутатор устройств U8 74HC245
  
  // Обрабатываем события
  for(int bank = 0; bank < NUM_BANKS; bank++) {
    
     byte diff;
    
    
    diff = banks[bank].top ^ prev_banks[bank].top;
    if(diff) {
      for(int key = 0; key < 8; key++) {
        if(bitRead(diff, key)) { 
          event_t event = bitRead(banks[bank].top, key) ? KEY_UP : KEY_PRESSED;
          trigger(&keys[bank * 8 + key], event);
        }
      }
    }
    
    
    diff = banks[bank].bottom ^ prev_banks[bank].bottom;
    if(diff) {
      for(int key = 0; key < 8; key++) {
        if(bitRead(diff, key)) { 
          event_t event = bitRead(banks[bank].bottom, key) ? KEY_RELEASED : KEY_DOWN;           
          trigger(&keys[bank * 8 + key], event);
        }
      }
    }
    
  }

}

void loop() {
  static uint32_t tmr1 = 0; //таймер опроса клавиатуры
  static uint32_t tmr2 = 0; //таймер опроса аналоговых устройств
  if (millis() - tmr1 >= 10) {  
    tmr1 = millis();            
    scan();
    increment();
  }
  if (millis() - tmr2 >= 100) { 
    tmr2 = millis();            
    footpedal();
  }
 
}